/********************************** (C) COPYRIGHT *******************************
* File Name          : usb_host_app.c
* Author             : kingsley
* Version            : V1.0
* Date               : 2020/05/22
* Description 		 : USB host app with tmos
*******************************************************************************/



#include "CH57x_common.h"
#include "usb_host_app.h"
#include "CH57xBLE_LIB.H"


//USB
#define USB_LOG PRINT
//#define USB_LOG 

#define USB_HOST_APP_CHECK_TICK_EVT_TIME         MS1_TO_SYSTEM_TIME(50)
#define USB_HOST_APP_USB_IN_EVT_TIME             MS1_TO_SYSTEM_TIME(60)

//handler for callback
static usb_host_app_event_handler_t m_handlers;

//task id
static uint8_t usb_host_app_task_id = INVALID_TASK_ID;

//usb out in buffer
__align(4) uint8_t  RxBuffer[ MAX_PACKET_SIZE ] ;      // IN, must even address
__align(4) uint8_t  TxBuffer[ MAX_PACKET_SIZE ] ;      // OUT, must even address


uint8_t tmpBuf[11] = {0x53,0x4E,0x08,0x00,0x23,0x01,0x53,0x49,0x4e,0x4F,0x65};
uint8_t RX_BUF[11];


/*发送数据长度*/
uint8_t	UsbOut( uint8_t *pBuf,uint8_t len )  // 中断端点3发送
{
    uint8_t TxCnt,s,endp;
    endp = ThisUsbDev.GpVar[1];
    while ( len ) {
        //mDelayuS( 200 );
        R8_UH_TX_LEN = len >= 32 ? 32 : len;
        for ( TxCnt = 0; TxCnt != R8_UH_TX_LEN; TxCnt ++ ) {
            TxBuffer[ TxCnt ] = *pBuf;
            pBuf ++;
        }
        s = USBHostTransact( USB_PID_OUT << 4 |( endp & 0x7F), endp & 0x80 ? RB_UH_R_TOG | RB_UH_T_TOG : 0, 0 );  // OUT数据
        if ( s == ERR_SUCCESS ) {
            len -= R8_UH_TX_LEN;
            endp ^= 0x80;  // 同步标志翻转
            ThisUsbDev.GpVar[1] = endp;  // 保存同步标志位
            
        }
        else return( s );
    }
    return ERR_SUCCESS;
}

/*返回数据接收长度，数据存放RxBuffer*/
uint8_t	UsbBulkIn( void )
{
    uint8_t i,s,len,endp = ThisUsbDev.GpVar[0];
    s = USBHostTransact( USB_PID_IN << 4 | (endp & 0x7F), endp & 0x80 ? RB_UH_R_TOG | RB_UH_T_TOG : 0, 0 );
    if ( s == ERR_SUCCESS ) {
        endp ^= 0x80;  // 同步标志翻转
        ThisUsbDev.GpVar[0] = endp;  // 保存同步标志位
        //tmos_memcpy(RX_BUF,RxBuffer,11);
        len = sizeof(RX_BUF);  // 接收到的数据长度
        if ( len ) {
            if(m_handlers)
            {
                usb_host_app_event_t event;
                event.type = USB_HOST_APP_DATA_IN;
                event.data.p_data = RxBuffer;
                //tmos_memcpy(event.data.p_data,RxBuffer,len);
                event.data.bytes = len;
                m_handlers(&event);
            }
//            USB_LOG("CH579 In data: \n");
//            for ( i = 0; i < len; i ++ ) USB_LOG("%02X ",(UINT16)(RxBuffer[i]) );
//            USB_LOG("\n");
        }
        return len;
    }
    else if ( s != ( USB_PID_NAK | ERR_USB_TRANSFER ) ) USB_LOG(" error %02x\n",(UINT16)s);  // 可能是断开了
    return 0;
}




uint8_t	usb_host_app_out( uint8_t *pBuf,uint8_t len )  // 中断端点3发送
{
    if(len>sizeof(tmpBuf)){
        return FALSE;
    }
    
    tmos_memcmp(tmpBuf,pBuf,len);
    tmos_set_event( usb_host_app_task_id, TMOS_USB_HOST_APP_OUT_EVT);
    //tmos_start_task( usb_host_app_task_id, TMOS_USB_HOST_APP_OUT_EVT,200);
    return TRUE;
}


/*********************************************************************
 * @fn      Peripheral_ProcessEvent
 *
 * @brief   Peripheral Application Task event processor.  This function
 *          is called to process all events for the task.  Events
 *          include timers, messages and any other user defined events.
 *
 * @param   task_id - The TMOS assigned task ID.
 * @param   events - events to process.  This is a bit map and can
 *                   contain more than one event.
 *
 * @return  events not processed
 */
uint16_t usb_host_app_process_event( uint8_t task_id, uint16_t events )
{

//  VOID task_id; // TMOS required parameter that isn't used in this function

    if ( events & SYS_EVENT_MSG ) {
        uint8_t *pMsg;

        if ( (pMsg = tmos_msg_receive( usb_host_app_task_id )) != NULL ) {
            //Peripheral_ProcessTMOSMsg( (tmos_event_hdr_t *)pMsg );
            // Release the TMOS message
            tmos_msg_deallocate( pMsg );
        }
        // return unprocessed events
        return (events ^ SYS_EVENT_MSG);
    }

    if ( events & TMOS_USB_HOST_APP_CHECK_TICK_EVT ) {
        // Start the Device
        if ( R8_USB_INT_FG & RB_UIF_DETECT ) {  // 如果有USB主机检测中断则处理
            R8_USB_INT_FG |= RB_UIF_DETECT ;
            
            uint8_t AnalyzeRootHub_result;
            AnalyzeRootHub_result = AnalyzeRootHub();
            switch(AnalyzeRootHub_result)
            {
                case ERR_USB_CONNECT:
                    USB_LOG("TMOS_USB_HOST_APP_INIT_ROOT_DEVICE_EVT\r\n");
                    tmos_start_task( usb_host_app_task_id, TMOS_USB_HOST_APP_INIT_ROOT_DEVICE_EVT, MS1_TO_SYSTEM_TIME(200) );
                    //tmos_start_task( usb_host_app_task_id, TMOS_USB_HOST_APP_TEST_EVT, MS1_TO_SYSTEM_TIME(200) );
                    //tmos_set_event( usb_host_app_task_id, TMOS_USB_HOST_APP_TEST_EVT);
                    
                    break;
                case ERR_USB_DISCON:
                    //tmos_stop_task( usb_host_app_task_id, TMOS_USB_HOST_APP_IN_EVT );
                    tmos_stop_task( usb_host_app_task_id, TMOS_USB_HOST_APP_INIT_ROOT_DEVICE_EVT);
                    USB_LOG("ERR_USB_DISCON\r\n");
                    if(m_handlers)
                    {
                        usb_host_app_event_t event;
                        event.type = USB_HOST_APP_DISCONNECTED_EVT;
                        
                        m_handlers(&event);
                    }
                    break;
                default:
                    USB_LOG("AnalyzeRootHub_result %02x\r\n",AnalyzeRootHub_result);
                    break;
            }
        }

        tmos_start_task( usb_host_app_task_id, TMOS_USB_HOST_APP_CHECK_TICK_EVT, USB_HOST_APP_CHECK_TICK_EVT_TIME );
        return ( events ^ TMOS_USB_HOST_APP_CHECK_TICK_EVT );
    }
    if(events & TMOS_USB_HOST_APP_IN_EVT)
    {
        USB_LOG("start in\r\n");
        UsbBulkIn();
        USB_LOG("start in2\r\n");
        UsbBulkIn();
        //tmos_start_task( usb_host_app_task_id, TMOS_USB_HOST_IN_EVT, USB_HOST_APP_USB_IN_EVT_TIME );
        return ( events ^ TMOS_USB_HOST_APP_IN_EVT );
    }
    if(events & TMOS_USB_HOST_APP_OUT_EVT)
    {
        //USB_LOG("START OUT\r\n");
        uint8_t i;
        for(i=0;i<11;i++)
        {
            USB_LOG("%02X ",tmpBuf[i]);
        }
        uint8_t result = UsbOut( tmpBuf,11 );
        if(ERR_SUCCESS !=result )
        {
            USB_LOG("RETRY %02x\r\n",result);
            tmos_start_task( usb_host_app_task_id, TMOS_USB_HOST_APP_OUT_EVT, 80 );
        }
        //USB_LOG("start in\r\n");
        UsbBulkIn();
        USB_LOG("start in2\r\n");
        UsbBulkIn();
        
        //tmos_start_task( usb_host_app_task_id, TMOS_USB_HOST_APP_IN_EVT, USB_HOST_APP_USB_IN_EVT_TIME );
        //tmos_set_event( usb_host_app_task_id, TMOS_USB_HOST_APP_IN_EVT );
        return ( events ^ TMOS_USB_HOST_APP_OUT_EVT );
    }
    //This event is for test only ,can be removed in release verison
    if(events & TMOS_USB_HOST_APP_TEST_EVT){
        USB_LOG("USB_HOST_APP_TEST_EVT\r\n");
        
        uint8_t result;
        result = InitRootDevice();
        
//                    FoundNewDev = 0;
//            mDelaymS( 200 );  // 由于USB设备刚插入尚未稳定,故等待USB设备数百毫秒,消除插拔抖动
//            s = InitRootDevice();  // 初始化USB设备
        if ( ERR_SUCCESS != result ) //	return( s );
        {
            USB_LOG("ERR_SUCCESS != InitRootDevice_result\r\n");
        }
            
        if(ThisUsbDev.DeviceStatus == ROOT_DEV_SUCCESS)
        {
            //result = usb_host_app_out( tmpBuf,11 );
            result = UsbOut( tmpBuf,11 );
            //mDelaymS( 2 );
            if(ERR_SUCCESS == result)
            {
                UsbBulkIn(  );
                //DelayMs(20);
                UsbBulkIn(  );  // 端点2批量接收
            }
        }
        
        return ( events ^ TMOS_USB_HOST_APP_TEST_EVT );
        
    }
    
    
        if(events & TMOS_USB_HOST_APP_INIT_ROOT_DEVICE_EVT)
        {
         
            USB_LOG("EVT TMOS_USB_HOST_APP_INIT_ROOT_DEVICE_EVT\r\n");
            uint8_t result;
            result = InitRootDevice();
            if(ERR_SUCCESS == result)
            {
                if(m_handlers)
                {        
                     usb_host_app_event_t event;
                     event.type = USB_HOST_APP_CONNECTED_EVT;
                            
                     m_handlers(&event);
                }
            }else{
                 tmos_start_task( usb_host_app_task_id, TMOS_USB_HOST_APP_INIT_ROOT_DEVICE_EVT, MS1_TO_SYSTEM_TIME(200) );
            }
            return (events ^ TMOS_USB_HOST_APP_INIT_ROOT_DEVICE_EVT);            
        }
        



    // Discard unknown events
    return 0;
}


//this function must be add after tmos inited
void usb_host_app_init(usb_host_app_event_handler_t event_handler)
{
	//usb ext osc
	SetSysClock( CLK_SOURCE_HSE_32MHz );
	//enable PLL for usb 
    PWR_UnitModCfg( ENABLE, UNIT_SYS_PLL );		
	
	//    
	pHOST_RX_RAM_Addr = RxBuffer;
    pHOST_TX_RAM_Addr = TxBuffer;
	
	//usb host init
	USB_HostInit();
	USB_LOG("usb host init\r\n");
	

	// register usb app task 
	usb_host_app_task_id = TMOS_ProcessEventRegister( usb_host_app_process_event );
	USB_LOG("usb_host_app_task_id=%d\r\n",usb_host_app_task_id);
    

    m_handlers = event_handler;
    
	//start tick usb check for every 10ms 
	tmos_start_task( usb_host_app_task_id, TMOS_USB_HOST_APP_CHECK_TICK_EVT, USB_HOST_APP_CHECK_TICK_EVT_TIME );
   
	USB_LOG("waiting for device \r\n");
	
}
#ifdef USB_TEST
int main()
{
    uint8_t	s;

    SetSysClock( CLK_SOURCE_HSE_32MHz );
    PWR_UnitModCfg( ENABLE, UNIT_SYS_PLL );		// 打开PLL
    DelayMs(5);

    GPIOA_SetBits(GPIO_Pin_9);
    GPIOA_ModeCfg(GPIO_Pin_8, GPIO_ModeIN_PU);
    GPIOA_ModeCfg(GPIO_Pin_9, GPIO_ModeOut_PP_5mA);
    UART1_DefInit();
    USB_LOG( "Start @ChipID=%02X\n", R8_CHIP_ID );

    pHOST_RX_RAM_Addr = RxBuffer;
    pHOST_TX_RAM_Addr = TxBuffer;
    USB_HostInit();
    USB_LOG( "Wait Device In\n" );
	
    while(1)
    {
        s = ERR_SUCCESS;
        if ( R8_USB_INT_FG & RB_UIF_DETECT ) {  // 如果有USB主机检测中断则处理
            R8_USB_INT_FG = RB_UIF_DETECT ;
            s = AnalyzeRootHub( );
            if ( s == ERR_USB_CONNECT ) 		FoundNewDev = 1;
        }

        if ( FoundNewDev || s == ERR_USB_CONNECT ) {  // 有新的USB设备插入
            FoundNewDev = 0;
            mDelaymS( 200 );  // 由于USB设备刚插入尚未稳定,故等待USB设备数百毫秒,消除插拔抖动
            s = InitRootDevice();  // 初始化USB设备
//			if ( s != ERR_SUCCESS ) 	return( s );
            if(ThisUsbDev.DeviceStatus == ROOT_DEV_SUCCESS)
            {
                s = UsbOut( tmpBuf,11 );
                if(s == ERR_SUCCESS)
                {
                    UsbBulkIn(  );
                    UsbBulkIn(  );  // 端点2批量接收
                }

            }
        }
    }
}
#endif
